'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = explodeModule;
const t = require('babel-types');

let isProgram = value => {
  return value.type === 'Program';
};

let unexpected = value => {
  return new Error(`Unexpected node type: ${value.type}`);
};

let location = node => {
  return {
    start: node.start,
    end: node.end,
    loc: node.loc
  };
};

let comments = node => {
  return {
    leadingComments: node.leadingComments,
    trailingComments: node.trailingComments,
    innerComments: node.innerComments
  };
};

let toVariableDeclaration = (kind, name, declaration) => {
  return Object.assign({}, t.variableDeclaration(kind, [Object.assign({}, t.variableDeclarator(t.identifier(name), t.toExpression(declaration)), location(declaration))]), location(declaration), comments(declaration));
};

let toModuleSpecifier = (local, external, source) => {
  let specifier = {};
  if (local) specifier.local = local;
  if (external) specifier.external = external;
  if (source) specifier.source = source;
  return specifier;
};

let getSource = node => {
  return node.source ? node.source.value : null;
};

let exploders = {};

exploders.ImportDeclaration = (node, exploded) => {
  let source = getSource(node);

  if (node.specifiers.length) {
    for (let specifier of node.specifiers) {
      let local = null;
      let external = null;

      if (specifier.type === 'ImportSpecifier') {
        external = specifier.imported.name;
        local = specifier.local.name;
      } else if (specifier.type === 'ImportDefaultSpecifier') {
        external = 'default';
        local = specifier.local.name;
      } else if (specifier.type === 'ImportNamespaceSpecifier') {
        local = specifier.local.name;
      } else {
        throw unexpected(specifier);
      }

      exploded.imports.push(toModuleSpecifier(local, external, source));
    }
  } else {
    exploded.imports.push(toModuleSpecifier(null, null, source));
  }
};

exploders.ExportDefaultDeclaration = (node, exploded) => {
  let source = getSource(node);
  let declaration = node.declaration;
  let local;

  if (t.isIdentifier(declaration)) {
    local = declaration.name;
  } else if (t.isFunctionDeclaration(declaration) || t.isClassDeclaration(declaration)) {
    if (declaration.id) {
      local = declaration.id.name;
    } else {
      local = '_default';
      declaration = toVariableDeclaration('const', local, declaration);
    }

    exploded.statements.push(declaration);
  } else if (t.isExpression(declaration)) {
    local = '_default';
    exploded.statements.push(toVariableDeclaration('const', local, declaration));
  } else {
    throw unexpected(declaration);
  }

  exploded.exports.push(toModuleSpecifier(local, 'default', source));
};

exploders.ExportNamedDeclaration = (node, exploded) => {
  let source = getSource(node);
  let declaration = node.declaration;

  if (declaration) {
    if (t.isVariableDeclaration(declaration)) {
      for (let declarator of declaration.declarations) {
        let name = declarator.id.name;
        exploded.exports.push(toModuleSpecifier(name, name, source));
        exploded.statements.push(Object.assign({}, t.variableDeclaration(declaration.kind, [declarator]), location(declarator), comments(declaration)));
      }
    } else if (t.isClassDeclaration(declaration) || t.isFunctionDeclaration(declaration) || t.isTypeAlias(declaration) || t.isInterfaceDeclaration(declaration)) {
      let name = declaration.id.name;
      exploded.exports.push(toModuleSpecifier(name, name, source));
      exploded.statements.push(declaration);
    } else {
      throw unexpected(declaration);
    }
  } else {
    for (let specifier of node.specifiers) {
      let exported = specifier.exported.name;
      let local;
      let external;

      if (specifier.local) {
        local = specifier.local.name;
        external = exported;
      } else {
        local = exported;
        external = exported;
      }

      exploded.exports.push(toModuleSpecifier(local, external, source));
    }
  }
};

exploders.ExportAllDeclaration = (node, exploded) => {
  let source = getSource(node);
  exploded.exports.push(toModuleSpecifier(null, null, source));
};

exploders.VariableDeclaration = (node, exploded) => {
  for (let declarator of node.declarations) {
    exploded.statements.push(Object.assign({}, t.variableDeclaration(node.kind, [declarator]), location(declarator), comments(node)));
  }
};

// do nothing...
exploders.EmptyStatement = () => {};

function explodeModule(node) {
  if (t.isFile(node)) {
    node = node.program;
  } else if (!t.isProgram(node)) {
    throw new Error(`Must pass a "File" or "Program" node to explode module, received ${node.type}`);
  }

  let exploded = {
    imports: [],
    exports: [],
    statements: []
  };

  for (let item of node.body) {
    let exploder = exploders[item.type];
    if (exploder) {
      exploder(item, exploded);
    } else {
      exploded.statements.push(item);
    }
  }

  return exploded;
}